/*
	garith.c -- A GNOME Arithmetic test
		Version 2.1
*/

/*

This and all other files included with this distribution are
Copyright 1997 Micah Stetson.  They are fully redistributable
under the terms of the GNU General Public License. For details,
please see the COPYING file which you should have recieved with
this file.

*/

/*
	This program has probably not been put together in the
	most efficient way. If you see areas in which it may be
	improved, feel free to suggest such improvement to me or
	if you would rather improve it yourself, just send
	me a patch. I will, more than likely, include these
	suggestions/patches in the main distribution and give you
	credit for them in the README and HISTORY files.
*/

#include <gtk/gtk.h>		/* GTK stuff */
#include <gnome.h>		/* GNOME stuff */
#include <stdlib.h>
#include <time.h>		/* we use time as seed for rand() */

/*
	Global variables that need to be accessed by multiple
	functions.  If I/you can figure out a good way to eliminate
	some/all of these, it ought to be done.
*/

	GtkWidget	*main_window;
	GtkWidget	*pref_window;
	GtkWidget	*pixmap;
	GtkWidget	*right_pct;
	GtkWidget	*complete_num;
	GtkWidget	*speed_num;
	GtkWidget	*probnum1;
	GtkWidget	*probop;
	GtkWidget	*probnum2;
	GtkWidget	*difficulty;

/*
	XPM variables to hold XPM data for X Arith logo,
	the 'RIGHT!' pixmap, and the 'WRONG!' pixmap.
	The actual data is at the end of this file.
	I think I'm going to move the pictures out of the
	source and into their own files, this will be easier
	to maintain.  I need a better G Arith logo, this one
	was made when it was called X Arith and used XForms.
	XForms has a darker default background color on which
	the white text of the logo shows up quite well; it
	doesn't show up well at all though in GTK so I need to
	fix it.
*/

static gchar * garith_logo[];
static gchar * right_xpm[];
static gchar * wrong_xpm[];

/*
	Global variables for the operation, the two 
	factors (or whatever), how many correct 
	answers were given, problems completed, the 
	integer that the factors must be less than, 
	and the starting time of the current test 
	(reset_score() sets this).  I haven't finished
	implementing the timer; I may use a gtimer for
	this.
*/

gulong		num1, num2;
guint		right = 0, done = 0;
gldouble	less_than;
gchar		op;
time_t		starttime;

/*
	This function gets a pixmap from a variable (like those at
	the end of the file) and returns it as a GdkPixmap. Settings
	for background color, etc. are set to those of the window
	of the supplied GtkWidget.

	Much of this was taken from get_monitor_preview_widget() in
	main.c of GNOME's desktop_properties program, thanks.
*/

GdkPixmap* get_pixmap_from_var(gchar **xpmvar, GtkWidget *window) {
	GdkPixmap *xpm, *mask;
	GtkStyle  *style;

	style = gtk_widget_get_style (window);
	xpm = gdk_pixmap_create_from_xpm_d (window->window, &mask,
		&style->bg [GTK_STATE_NORMAL], xpmvar);

	return xpm;
}

/*
	This function gets a pixmap from a variable (like those at
	the end of the file) and returns it's mask as a GdkPixmap.
	Settings for background color, etc. are set to those of the
	window of the supplied GtkWidget.

	Perhaps this function and the one above should be consolidated
	into one function that returns a pointer to a structure
	containing both the pixmap and the mask.  Is there a better way
	to do this?

	Again, much of this was taken from get_monitor_preview_widget()
	in main.c of GNOME's desktop_properties program.
*/

GdkPixmap* get_mask_from_var(gchar **xpmvar, GtkWidget *window) {
	GdkPixmap *xpm, *mask;
	GtkStyle  *style;

	style = gtk_widget_get_style (window);
	xpm = gdk_pixmap_create_from_xpm_d (window->window, &mask,
		&style->bg [GTK_STATE_NORMAL], xpmvar);

	return mask;
}

/*
	This function sets problem characteristics and displays
	a randomly generated problem. It also handles display of
	the options window. This should probably be moved to a
	different function that would handle all windows.  But as
	the options window is the only subwindow at this point and
	as this function handles all interaction with that window,
	it stays here for now.  I may move it in the next version.
*/

void set_problem(GtkWidget *widget, gpointer *data) {
	gint		digits, c;
	gchar		string[10];

	switch((gint) data) {
		case 0:	/* New Problem */
			/* I don't like this below, but I don't know
				a more efficient way to do it.  This
				(or something that serves the same
				purpose) is necessary in order to
				ensure that our answers are always
				integers.  Perhaps the higher levels
				should have floating point answers,
				but at least the lower levels (3 or 4
				and below) should not, IMHO. */
			if (op == '/') {
				num1 = (gulong) (less_than * rand()
					/ (RAND_MAX+1.0));
				if (num1 > (less_than / 10.0))
					num2 = 1 + (gulong) (10.0 * rand()
						/ (RAND_MAX+1.0));
				else if (num1 > (less_than / 100.0))
					num2 = 1 + (gulong) (100.0 * rand()
						/ (RAND_MAX+1.0)); 
				else if (num1 > (less_than / 1000.0))
					num2 = 1 + (gulong) (1000.0 * rand()
						/ (RAND_MAX+1.0)); 
				else if (num1 > (less_than / 10000.0))
					num2 = 1 + (gulong) (10000.0 * rand()
						/ (RAND_MAX+1.0)); 
				else if (num1 > (less_than / 100000.0))
					num2 = 1 + (gulong) (100000.0 * rand()
						/ (RAND_MAX+1.0)); 
				else if (num1 > (less_than / 1000000.0))
					num2 = 1 + (gulong) (1000000.0
						* rand() / (RAND_MAX+1.0)); 
				else if (num1 > (less_than / 10000000.0))
					num2 = 1 + (gulong) (10000000.0
						* rand() / (RAND_MAX+1.0)); 
				else if (num1 >= 1)
					num2 = 1 + (gulong) (100000000.0
						* rand() / (RAND_MAX+1.0));
				else if (num1 == 0)
					num2 = 1 + (gulong) (less_than
						* rand() / (RAND_MAX+1.0));
			} else {
				num1 = (gulong) (less_than * rand()
					/ (RAND_MAX+1.0));
				num2 = (gulong) (less_than * rand()
					/ (RAND_MAX+1.0));
			}

			if (op == '/') {
				sprintf(string, "%lu", (num1 * num2));
				gtk_label_set(GTK_LABEL (probnum1), string);

				sprintf(string, "%lu", num2);
				gtk_label_set(GTK_LABEL (probnum2), string);
			} else if (op == '-' && num1 < num2) {
				sprintf(string, "%lu", num2);
				gtk_label_set(GTK_LABEL (probnum1), string);

				sprintf(string, "%lu", num1);
				gtk_label_set(GTK_LABEL (probnum2), string);
			} else {
				sprintf(string, "%lu", num1);
				gtk_label_set(GTK_LABEL (probnum1), string);

				sprintf(string, "%lu", num2);
				gtk_label_set(GTK_LABEL (probnum2), string);
			}

			sprintf(string, "%c", op);
			gtk_label_set(GTK_LABEL (probop), string);

			break;
		case 1:		/* Digits */
			c = 0;
			digits = (gint) GTK_ADJUSTMENT (widget)->value;

			for(less_than = 1.0; c < digits; c++) {
				less_than *= 10.0;
			}
			/* The first arg here is meaningless */
			set_problem(probnum1, 0);
			break;
		case 2:		/* Addition */
			op = '+';
			/* Again, arg1 below is meaningless */
			set_problem(probop, 0);
			break;
		case 3:		/* Subtraction */
			op = '-';
			/* Again, arg1 below is meaningless */
			set_problem(probop, 0);
			break;
		case 4:		/* Multiplication */
			op = '*';
			/* Again, arg1 below is meaningless */
			set_problem(probop, 0);
			break;
		case 5:		/* Division */
			op = '/';
			/* Again, arg1 below is meaningless */
			set_problem(probop, 0);
			break;
		case 6:		/* Options Button */
			gtk_widget_show(pref_window);
			break;
		case 7:		/* Done Button */
			/* Closes pref_window */
			gtk_widget_hide(pref_window);

			break;
	}	

	return;
}

/*
	This function is called when the user enters a solution
	to the displayed problem.  It checks the answer, increments
	the number stored in right (if the user was right), displays
	a pixmap indicating whether the user's answer was correct or
	incorrect, increments the number of completed problems (done), 
	recalculates the percentage of correct problems, updates the 
	score display, and calls set_problem to display a new problem.
*/

void solve_prob(GtkWidget *widget) {
	gint		answer = 0;
	gint		solution = 0;
	time_t		now;
	gdouble		elapsed;
	gchar		string[10];
	GdkPixmap	*xpm, *mask;

	/* Get user's answer */

	answer = atoi(gtk_entry_get_text(GTK_ENTRY (widget)));

	/* Figure out the real answer */

	switch(op) {
		case '+':
			solution = num1 + num2;
			break;
		case '-':
			solution = abs(num1 - num2);
			break;
		case '*':
			solution = num1 * num2;
			break;
		case '/':
			solution = num1;
			break;
	}

	if (answer == solution) {	/* User was right */
		xpm = get_pixmap_from_var(right_xpm, main_window);
		mask = get_mask_from_var(right_xpm, main_window);
		gtk_pixmap_set(GTK_PIXMAP (pixmap), xpm, mask);
		right++;

	} else {			/* User was wrong */
		xpm = get_pixmap_from_var(wrong_xpm, main_window);
		mask = get_mask_from_var(wrong_xpm, main_window);
		gtk_pixmap_set(GTK_PIXMAP (pixmap), xpm, mask);
	}

	done++;
	sprintf(string, "%d", done);
	gtk_label_set(GTK_LABEL (complete_num), string);

	sprintf(string, "%d%%",
		(int) (100.0 * ((float) right / (float) done)));
	gtk_label_set(GTK_LABEL (right_pct), string);

	/* Figure average seconds per problem, and display */
	now = time(NULL);
	/* This does not work, need to read the difftime man page
		again.  Of course, I won't need to if I use a
		gtimer; need to read about that. */
	elapsed = difftime(starttime, now);
	sprintf(string, "%f", (float) elapsed / (float) done);

	set_problem(widget, 0);		/* first arg meaningless */	
	gtk_entry_set_text(GTK_ENTRY (widget), "");

	return;
}

/*
	This function resets the displays right_num and wrong_num to
	0 and calls set_problem to display a new one.  Also resets
	global variables pertaining to score.
*/

void reset_score(GtkWidget *widget, gpointer *data) {
	gchar		string[5];
	GdkPixmap	*xpm, *mask;

	/* Reset number right */
	right = 0;
	sprintf(string, "%d%%", 100);
	gtk_label_set(GTK_LABEL (right_pct), string);

	/* Reset number done */
	done = 0;
	sprintf(string, "%d", 0);
	gtk_label_set(GTK_LABEL (complete_num), string);

	/* Reset timer display */
	/* not gotten to this yet */

	/* Reset pixmap to G Arith Logo */
	xpm = get_pixmap_from_var(garith_logo, main_window);
	mask = get_mask_from_var(garith_logo, main_window);
	gtk_pixmap_set(GTK_PIXMAP (pixmap), xpm, mask);

	/* Produce new problem */
	set_problem(widget, (gpointer) 0);

	/* Reset timer */
	starttime = time(NULL);

	return;
}

void quit_confirm(GtkWidget *widget, gpointer *data)
{
	gtk_main_quit();
}

/*
	This function creates all the widgets used in the program.
	It is separate because it makes the source more readable for me.
*/

gint make_widgets(void)
{
	GtkWidget	*widget1;
	GtkWidget	*widget2;
	GtkWidget	*box1;
	GtkWidget	*box2;
	GtkWidget	*box3;
	GtkWidget	*box4;
	GtkWidget	*entry;
	GtkObject	*adjustment;
	GdkPixmap	*xpm, *mask;

	/* Begin main_window */
	main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW (main_window),
			"GNOME Arithmetic");
		gtk_container_border_width (GTK_CONTAINER (main_window), 10);
		gtk_signal_connect(GTK_OBJECT (main_window), "destroy",
					(GtkSignalFunc) quit_confirm, NULL);

	box1 = gtk_vbox_new(FALSE, 10);
		gtk_container_add(GTK_CONTAINER (main_window), box1);
		gtk_widget_show(box1);

	/*
	The solution to the displayed problem is to be typed into
	this entry widget.  It is created up here so that it
	can be the arguement to solve_prob() which is called
	by the button housing the logo.  The reason for this
	nonsense is that I can find no better way to have
	solve_prob() called _only_ when the user hits [enter].
	This should be fixed, if possible, because I don't like
	having the logo in a button.
	*/

	/* Logo/Right/Wrong pixmap and its frame */
	widget1 = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
		gtk_box_pack_start(GTK_BOX (box1), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);
/*	widget2 = gtk_button_new();
		gtk_signal_connect_object(GTK_OBJECT (widget2), "clicked",
			(GtkSignalFunc) solve_prob, GTK_OBJECT (entry));
		gtk_container_add(GTK_CONTAINER (widget1), widget2);
		GTK_WIDGET_SET_FLAGS (widget2, GTK_CAN_DEFAULT);
		gtk_widget_grab_default(widget2);
		gtk_widget_show(widget2); */
	widget2 = gtk_frame_new(NULL);
		gtk_container_add(GTK_CONTAINER (widget1), widget2);
		gtk_widget_show(widget2);
	xpm = get_pixmap_from_var(garith_logo, main_window);
	mask = get_mask_from_var(garith_logo, main_window);
	pixmap = gtk_pixmap_new(xpm, mask);
		gtk_container_add(GTK_CONTAINER (widget2), pixmap);
		gtk_widget_show(pixmap);

	/* Problem/Solution/Score Area */
	box2 = gtk_hbox_new(TRUE, 10);
		gtk_box_pack_start(GTK_BOX (box1), box2, TRUE, FALSE, 0);
		gtk_widget_show(box2);
	/* Problem/Solution Area */
	widget1 = gtk_frame_new("Problem");
		gtk_box_pack_start(GTK_BOX (box2), widget1, TRUE, FALSE, 0);
		gtk_widget_show(widget1);
	box3 = gtk_vbox_new(TRUE, 0);
		gtk_container_border_width (GTK_CONTAINER (box3), 10);
		gtk_container_add(GTK_CONTAINER (widget1), box3);
		gtk_widget_show(box3);
	box4 = gtk_hbox_new(FALSE, 10);
		gtk_box_pack_start(GTK_BOX (box3), box4, TRUE, FALSE, 0);
		gtk_widget_show(box4);
	probnum1 = gtk_label_new("0");
		gtk_box_pack_end(GTK_BOX (box4), probnum1, FALSE, FALSE, 0);
		gtk_widget_show(probnum1);
	box4 = gtk_hbox_new(FALSE, 10);
		gtk_box_pack_start(GTK_BOX (box3), box4, TRUE, FALSE, 0);
		gtk_widget_show(box4);
	probnum2 = gtk_label_new("0");
		gtk_box_pack_end(GTK_BOX (box4), probnum2, FALSE, FALSE, 0);
		gtk_widget_show(probnum2);
	probop = gtk_label_new("+");
		gtk_box_pack_end(GTK_BOX (box4), probop, FALSE, FALSE, 0);
		gtk_widget_show(probop);
	widget1 = gtk_hseparator_new();
		gtk_box_pack_start(GTK_BOX (box3), widget1, TRUE, FALSE, 0);
		gtk_widget_show(widget1);
	entry = gtk_entry_new();
		gtk_signal_connect_object(GTK_OBJECT (entry), "activate",
			(GtkSignalFunc) solve_prob, GTK_OBJECT (entry));
		gtk_box_pack_start(GTK_BOX (box3), entry, TRUE, FALSE, 0);
		gtk_widget_show(entry);

	/* Score display */
	widget1 = gtk_frame_new("Score");
		gtk_box_pack_start(GTK_BOX (box2), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);
	box3 = gtk_vbox_new(TRUE, 10);
		gtk_container_border_width (GTK_CONTAINER (box3), 10);
		gtk_container_add(GTK_CONTAINER (widget1), box3);
		gtk_widget_show(box3);
	box4 = gtk_hbox_new(TRUE, 10);
		gtk_box_pack_start(GTK_BOX (box3), box4, TRUE, FALSE, 0);
		gtk_widget_show(box4);
	widget1 = gtk_frame_new("Right");
		gtk_frame_set_label_align(GTK_FRAME (widget1), 0.5, 0.0);
		gtk_box_pack_start(GTK_BOX (box4), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);
	right_pct = gtk_label_new("100%");
		gtk_container_add(GTK_CONTAINER (widget1), right_pct);
		gtk_widget_show(right_pct);
	widget1 = gtk_frame_new("Completed");
		gtk_frame_set_label_align(GTK_FRAME (widget1), 0.5, 0.0);
		gtk_box_pack_start(GTK_BOX (box4), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);
	complete_num = gtk_label_new("0");
		gtk_container_add(GTK_CONTAINER (widget1), complete_num);
		gtk_widget_show(complete_num);
	widget1 = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
		gtk_box_pack_start(GTK_BOX (box3), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);
	widget2 = gtk_frame_new("Avg. Speed");
		gtk_frame_set_label_align(GTK_FRAME (widget2), 0.5, 0.0);
		gtk_container_add(GTK_CONTAINER (widget1), widget2);
		gtk_widget_show(widget2);
	speed_num = gtk_label_new("0");
		gtk_container_add(GTK_CONTAINER (widget2), speed_num);
		gtk_widget_show(speed_num);
	widget1 = gtk_button_new_with_label("Reset Score");
		gtk_signal_connect(GTK_OBJECT (widget1), "clicked",
			(GtkSignalFunc) reset_score, NULL);
		gtk_box_pack_start(GTK_BOX (box3), widget1, TRUE, FALSE, 0);
		gtk_widget_show(widget1);

	/* Quit and Options buttons */
	box2 = gtk_hbox_new(TRUE, 10);
		gtk_box_pack_start(GTK_BOX (box1), box2, TRUE, FALSE, 0);
		gtk_widget_show(box2);
	widget1 = gtk_button_new_with_label("Options");
		gtk_signal_connect(GTK_OBJECT (widget1), "clicked",
			(GtkSignalFunc) set_problem, (gpointer) 6);
		gtk_box_pack_start(GTK_BOX (box2), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);
	widget1 = gtk_button_new_with_label("Quit");
		gtk_signal_connect(GTK_OBJECT (widget1), "clicked",
			(GtkSignalFunc) quit_confirm, (gpointer) 0);
		gtk_box_pack_end(GTK_BOX (box2), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);

	/* End main_window */

	/* Begin pref_window */
	pref_window = gtk_window_new(GTK_WINDOW_DIALOG);
		gtk_window_set_title(GTK_WINDOW (pref_window),
			"G Arith: Preferences");
		gtk_container_border_width (GTK_CONTAINER (pref_window), 10);
		gtk_signal_connect_object(GTK_OBJECT (pref_window), "destroy",
					(GtkSignalFunc) gtk_widget_hide,
					GTK_OBJECT (pref_window));

	box1 = gtk_vbox_new(FALSE, 10);
		gtk_container_add(GTK_CONTAINER (pref_window), box1);
		gtk_widget_show(box1);

	box2 = gtk_hbox_new(FALSE, 10);
		gtk_box_pack_start(GTK_BOX (box1), box2, FALSE, FALSE, 0);
		gtk_widget_show(box2);

	widget1 = gtk_notebook_new();
		gtk_notebook_set_tab_pos(GTK_NOTEBOOK (widget1), GTK_POS_TOP);
		gtk_box_pack_start(GTK_BOX (box2), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);

	box2 = gtk_hbox_new(TRUE, 10);
		gtk_container_border_width (GTK_CONTAINER (box2), 10);
	widget2 = gtk_label_new("Game Settings");
		gtk_notebook_append_page(GTK_NOTEBOOK (widget1), box2, widget2);
		gtk_widget_show(box2);
		gtk_widget_show(widget2);

	box3 = gtk_vbox_new(TRUE, 10);
		gtk_container_border_width (GTK_CONTAINER (box3), 10);
	widget2 = gtk_label_new("Colors/Fonts");
		gtk_notebook_append_page(GTK_NOTEBOOK (widget1), box3, widget2);
		gtk_widget_show(box3);
		gtk_widget_show(widget2);

	box4 = gtk_vbox_new(FALSE, 10);
		gtk_box_pack_start(GTK_BOX (box2), box4, TRUE, FALSE, 0);
		gtk_widget_show(box4);

	adjustment = gtk_adjustment_new(2.0, 1.0, 9.0, 1.0, 1.0, 1.0);
		gtk_signal_connect(GTK_OBJECT (adjustment), "value_changed",
			(GtkSignalFunc) set_problem, (gpointer) 1);

	difficulty = gtk_vscale_new(GTK_ADJUSTMENT (adjustment));
		gtk_scale_set_digits(GTK_SCALE (difficulty), 0);
		gtk_box_pack_start(GTK_BOX (box4), difficulty, FALSE, FALSE, 0);
		gtk_widget_show(difficulty);
	widget1 = gtk_label_new("Difficulty");
		gtk_box_pack_start(GTK_BOX (box4), widget1, FALSE, FALSE, 0);
		gtk_widget_show(widget1);

	box4 = gtk_vbox_new(TRUE, 10);
		gtk_box_pack_start(GTK_BOX (box2), box4, TRUE, FALSE, 0);
		gtk_widget_show(box4);

	widget1 = gtk_radio_button_new_with_label(NULL, "Addition");
		gtk_signal_connect(GTK_OBJECT (widget1), "clicked",
			(GtkSignalFunc) set_problem, (gpointer) 2);
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (widget1), TRUE);
		gtk_box_pack_start(GTK_BOX (box4), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);
	widget1 = gtk_radio_button_new_with_label(
			gtk_radio_button_group(GTK_RADIO_BUTTON (widget1)),
			"Subtraction");
		gtk_signal_connect(GTK_OBJECT (widget1), "clicked",
			(GtkSignalFunc) set_problem, (gpointer) 3);
		gtk_box_pack_start(GTK_BOX (box4), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);
	widget1 = gtk_radio_button_new_with_label(
		gtk_radio_button_group(GTK_RADIO_BUTTON (widget1)),
			"Multiplication");
		gtk_signal_connect(GTK_OBJECT (widget1), "clicked",
			(GtkSignalFunc) set_problem, (gpointer) 4);
		gtk_box_pack_start(GTK_BOX (box4), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);
	widget1 = gtk_radio_button_new_with_label(
		gtk_radio_button_group(GTK_RADIO_BUTTON (widget1)),
			"Division");
		gtk_signal_connect(GTK_OBJECT (widget1), "clicked",
			(GtkSignalFunc) set_problem, (gpointer) 5);
		gtk_box_pack_start(GTK_BOX (box4), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);

	widget1 = gtk_label_new("Not yet implemented.");
		gtk_box_pack_start(GTK_BOX (box3), widget1, TRUE, TRUE, 0);
		gtk_widget_show(widget1);

	box2 = gtk_hbox_new(FALSE, 10);
		gtk_box_pack_end(GTK_BOX (box1), box2, FALSE, FALSE, 0);
		gtk_widget_show(box2);

	widget1 = gtk_button_new_with_label("Done");
		gtk_signal_connect(GTK_OBJECT (widget1), "clicked",
			(GtkSignalFunc) set_problem, (gpointer) 7);
		gtk_box_pack_end(GTK_BOX (box2), widget1, FALSE, FALSE, 0);
		GTK_WIDGET_SET_FLAGS (widget1, GTK_CAN_DEFAULT);
		gtk_widget_grab_default(widget1);
		gtk_widget_show(widget1);

	/* End pref_window */

	return 0;
}

/*
	main(): Where all the fun begins.
*/

gint main(int argc, char *argv[])
{
	time_t now;
	gint seed;

	/* gtk_init() is called by all gtk apps.
	   gnome_init() is called by all gnome apps. */

	gtk_init(&argc, &argv);
	gnome_init(&argc, &argv);

	/* Make the windows */

	make_widgets();

	/* Set initial values for various options */

	now = time(NULL);
	seed = now;
	srand(seed);

	op = '+';
	less_than = 100;
	reset_score(right_pct, NULL);	/* The first arguement is meaningless */

	/* Show it */

	gtk_widget_show(main_window);

	/*
		Run it.
		Exit when the Quit button is pressed.
	*/
	gtk_main();
	return(0);
}

/* These below are soon to be moved into separate files */

/* XPM data for logo */
static gchar * garith_logo[] = {
/* width height ncolors chars_per_pixel */
"170 30 32 1",
/* colors */
"  c #000000",
". c #45454A",
"X c #838383",
"o c #5E5E62",
"O c #737373",
"+ c #4E4E52",
"@ c #D1D1D5",
"# c #EEEEEE",
"$ c #C9C9CD",
"% c #393939",
"& c #E6E6E6",
"* c #C1C1C5",
"= c #313131",
"- c #DEDEDE",
"; c #B4B4B4",
": c #87878B",
"> c #A4A4A4",
", c #77777B",
"< c #949494",
"1 c #6A6A6A",
"2 c #F2F2F6",
"3 c #3D3D41",
"4 c #5A5A5A",
"5 c #FFFFFF",
"6 c #252529",
"7 c #202020",
"8 c #A8A8AC",
"9 c #181818",
"0 s mask c none",
"q c #101010",
"w c #BDBDBD",
"e c #080808",
/* pixels */
"00000000000000000000000001 qqo00000000000000000000000000000007q4,=%0X=%3X0000000000000o00=+00%+000000000000000000000000000000000,q:0000X0000000000% 9=1000000=400000000000",
"00000000000000000000000:47 q,00000000000000000000000000000000%=<0=+00X:000000000000017e40,X006o000000000000000000000000000000000o=X00000000000000+=,,9400000X=400000000000",
"00000000000000000000000X6+0e+000000000000000000000000000000009+O+6O000000000000000O6q%qo00001=0000000000000000000000000000000000O900000000000000004<X7+0000006o00000000000",
"00000000000O..00000000000004300000000000000000000000000000000+97930000000000000000 %1O640000=..+0000000000000000000,00000000000014000000000X.730000X69eO0000X%100000000000",
"0000000000,79O00000000000000X0000000000000000000000000000000001+o00000000000000000%+0091000O=o.q00000000000000000o=.X000000000000000000000X67.6.000034==X0000+000000000000",
"000000000X=.0000000000000000000000000000000000000:11000000000000000000000000000000:,00=o000.e6=.0000000000000001===o0000000000000000000000o6,<.%0000001q000000000000000000",
"00000:000oqO000000000000000wwww0000000wwww000000O.q%00000000000000wwww000000000000000X7O001 9=o00000000000000X%==+:O3O0000000000000000:,00+9:,73000000%6:00000000000000000",
"00004=100.3463X000000000000255-0000000@55&0000O.991000000000000000555$00000000000000006100X%40000000000000000,%+X,=e3X0000000000000003e400466%qq1000o.%o000000000000000000",
"000. 3000.q7393X0000000000002550000000252000017qo0000000000000000$555&000000000000000,7O00000000000000000000000X%e%1X00000000000000. =oX0003 7o%=:07=q40000000000,Oo000000",
"001=O0000%e3XOq0000000000000w55200000&55w00001o000000000000000000&5255000000000000w--@;@-----w00000000w--@000000,<8wwwwwwwww@w0000@w8,00000468-@o00XO000000000000= 9=O0000",
"006300000Oe,0X610000000000000$55@O00-55$000000000000000000000000055w55@3O000000000055555555555-0,37+,00&5w0000055555555555555w0000*50O=%o00.9:@53000000000000000.6OO6%0000",
"00=.%6300O74,.6000000000000000-5500w55-0000000000000000000000000&5&:@553O000000000055wwwwwww-55;67+6+00-5w0000w&-----55-----&w0000w5oe%=eX04=7w510000000000000006.<<=6,000",
"0X97=%94004q77.0000000000000000&55w5520000000000000000000000000055;1455$000000000005500000000*5-7<0=+00-5w000000000005500000000000w51oX,6+001%w50000000000O+.0006+<X7eX000",
"007qoX.6000OoO0000000000000X,0O=5555500000000000000000000000000@520X3$55,000000000055000000X3O55=:,6300-5w000000000005500000000000w5><0:910000-5000000000O997%:0+q+7991000",
"003900.%:000000000000000003 %000;555w0000000000000000000000000055$00X+55@0000000000550000:% e<5#q73e610-5w000000000005500000000000w5w0:+9O0000-500000000X=3<1e00:3=%%9,000",
"00+64o=.0000000000000000. =9.000;555-0000000000000000000000000@520000o-55000000000055ww;:+O:X#5$= 74%70-5w000000000005500000000000w5&*w;w-----2500000000, 10o6,0000X1%0000",
"00X. =7O000000000000000794O3=00w5555500000000000000000:000000055@00000;55@00000000055555555555@049<04%:-5w00000000000550O+X0000000w5555555555555000OO000X7%+=q1000017+0000",
"0000Oo000000000000X1X004=00=+00252.555000000000000000X39%o000$55555555555500000000055www00&5w100==:O730-5w00000000000553==,0000000w5;X00000000-5004 .0000o e7=6100o%3X0000",
"0000000000000000003q.000,00.30255<915520000000000000:67o7%0002555555555555@0000000o55X000005&,00o97==,0-5w0000000000,55=.,0X000000w5w000000000-50O%=70000,6%0X710,73000000",
"00000000000000000=7O0000000=+-55<==.*55&000000000000O9,0+%00$55$wwwwwwww-5500000O9955000000*5$00043%O00-5w000000000,355O<3 1000000w5w000000000-50o%OqO000, 101910000000000",
"00000000000000001=O00000000%<55-,+,00&55@0000000,=40+7,,=600&5&000000000055@00001qO550000000&5000000000-5w0000000000455. 6oX000000w5w000000,.=$5000X%3000X7%3%.00000000000",
"0000000000000000+=0o,000000155&0000000255w0000X% q300eq=e7305500000000000@550000O=155%X0000OO5200000000-5w000000000005564,00000000w5w00000,97=;50000330000197=,00000000000",
"000000000X4=3X0066q76+0000055200000000w555000+ =o7.00+e71+q-5&00000000000055$000X= 5593000006@5@0000000@500000000000055,0000000000w5w000003=<X;50000o7,X000X,0000000000000",
"000000000. .9+003e3O==X000$*@-00000000w---@00=%O03.00Oq,014@--0000000O000w---w000+1@-Xq0000XO@--w00000080XX000000000$--w0000000000@-@0000093<<;$w000Xq==4000000000000000o=",
"00000000O61<4600.qo016,00000000000000000000001100=400+7,O==00000000003o00000000000X0046X00000000000001q+O=%0000000000000000000000000000000%7%%q=0000+97%100000000000000+ .",
"00000000oq:<%=00oqo0oq,00000000000000000000000000%+000q9=71000000000X=,0000000000000O73000000000000X0+600+%0000000000000000000000000000000Xq 7.7+003==o0000000000000000=o<",
"00000000O9=397o009=3=.00000000000000000000000000074000O43O000000000006O00000000000133=O0000000000+ %0O100.300000000000000000000000000000000%=<:.300110000000000000046,O.,0",
"000000000+  +==,0O3==X0000000000000000000000000006+00000000000000000O=000000000000.%%,0000000004e7q.0000X7o0000000000000000000000000000000093<Oq40000000000000000oqqqX0X<0",
"000000000O710O6X00000000000000000000000000000000:6400000000000000000O7:000000000000X000000000079+O%%000046430000000000000000000000000000000=6==%X00000000000000X99.+7O000<",
"000000000oq::36X0000000000000000000000000000000003O00000000000000000o6000000000000000000000000+=:<=+000034O X000000000000000000:1:000000000X3=6,000000000000000164:O6<000O"
};

/* XPM data for 'RIGHT!' */
static gchar *right_xpm[] = {
/* width height ncolors chars_per_pixel */
"170 30 32 1",
/* colors */
"  c #6060B4",
". c #5858C0",
"X c #0B0B71",
"o c #5A5A5B",
"O c #838383",
"+ c #919197",
"@ c #6F6F6F",
"# c #2E2E2F",
"$ c #0202FC",
"% c #3535DB",
"& c #1E1E8C",
"* c #0404AE",
"= c #414197",
"- c #19196C",
"; c #1F1F1F",
": c #7E7EB1",
"> c #4545CF",
", c #0B0B0B",
"< c #242473",
"1 c #333392",
"2 c #4D4D9E",
"3 c #2222E1",
"4 c #090951",
"5 c #46467C",
"6 c #1212F1",
"7 c #383838",
"8 s mask c none",
"9 c #1010B6",
"0 c #303075",
"q c #454544",
"w c #1B1BBD",
"e c #6A6ABB",
/* pixels */
"888888888888888888888888+@,,;@8888888888888888888888888888888;,oO778O#7qO8888888888888@O+7o8+7o888888888888888888888888888888888O;O8888OO88888888+7,;#@88888+7o88888888888",
"88888888888888888888888Oo;,;O8888888888888888888888888888888+77+87o+8OO8888888888888@;,o+OO88#o888888888888888888888888888888888o#O88888888888888o#OO;o88888O#o88888888888",
"888888888888++8888..ee8O<1e41 .eee+888888888+e888888888+:888+;o@o#@:e%%%%>e:88.e88@#;7,@8888@#+888888888888888888.:8888888888888@;>:8888888+2  88 =e.9w 8888+#@88888888888",
"8888888888+@qq8888>$$$$$$$$$$$$$$$$6.8888888:$$$$6%3$$$$%8888o;;;w$$$$$$$$$$$%6.8+,7$69w%%%%9w6688$3%%%eee.%%%888%62ee:88888888:21$e8888888 w$$$$$$$$$*@+888O7@88888888888",
"8888888888O;;@888888%$$$$$$$$$$$$$$$$6+88888883$$$$$$$$:888888@1$$$$$$$$$$$$$$$8887oe%$$$$$$$$w4+8:%$$$$$$$$3>88+3$$$$$$$$$$$$$$$$$e888888O#;w$$$$$$$$77O888+oO88888888888",
"888888888O7q88888888>$$$$$$$$$$$$$$$$$6+888888%$$$$$$$%8888888%$$$$$$$$$$$$$$$3888OO8:$$$$$$$$7qO88+$$$$$$$$e88@79$$$$$$$$$$$$$$$$$e888888o#O>$$$$$$$6@,+88888888888888888",
"8888+O+88@;@8+888888e$$$$$$$$$$$$$$$$$$6888888>$$$$$$$%888888%$$$$$$$$$$$$$$$$%888888O$$$$$$$$o+8888$$$$$$$$+O777w$$$$$$$$$$$$$$$$$e88OO8+q;+.$$$$$$$%7#O88888888888888888",
"888Oo7@88qqo#qO88888e$$$$$$$$$$$$$$$$$$$e8888+2$$$$$$$%88888e$$$$$$$$$$$$$$$$$>888888O9$$$$$$6888888$$$$$$$$8O7oO3$$$$$$$$$$$$$$$$$eOq,o88o##0$$$$$$$w7o888888888888888888",
"888q,q+8+q,;q;qO8888e$$$$$$$$$$$$$$$$$$$%8888@-$$$$$$$e888886$$$$$$$6%eee>%$$$e888888O9$$$$$$%888888$$$$$$$$8+OO7*$$$$$$$$$$$$$$$$$1,#oO88+q,;$$$$$$$-o+888888888O@o+88888",
"88@7@88887,qO@,+8888e$$$$$$$e8+.$$$$$$$$%8888@=$$$$$$$e8888.$$$$$$$%8@O++888e$e888888O9$$$$$$%888+++%$$$$$$$88O;73$$$%6$$$$$$$3%$$$<qO++888o#O3$$$$$$ 8888888888+7,;7@8888",
"8+#q+++88@,O8O#@8888e$$$$$$$+8883$$$$$$$%88888e$$$$$$$e88886$$$$$$%8O#7q@8888$:888888Ow$$$$$$%88Oq;o3$$$$$$688+OO$%+88e$$$$$$$888+69o@77o8+q;+3$$$$$$88888888888q#@@#78888",
"++#q7#q88@;oOq#O88888$$$$$$$888O$$$$$$$$.888888$$$$$$$e888+$$$$$$$++O;,,@8888.88888888%$$$$$$$$$$$$$$$$$$$$%888886:888+$$$$$$$8888.w,,77,O8o#;&$$$$$$8888888888+#q++##O888",
"+O;;77;o8+o,;;q+88888$$$$$$$88O0$$$$$$$$:888888$$$$$$$e888e$$$$$$%8@#,,#+8888888888888%$$$$$$$$$$$$$$$$$$$$%88888888888$$$$$$$88888O;oOO#o8+@7<$$$$$38888+@oq+8+#o+O;,O888",
"8+;,@Oq#88+@o@8888888$$$$$$$O+=$$$$$$$$%8888888$$$$$$$e888e$$$$$$e8Oq;;;o8888888888888%$$$$$$$$$$$$$$$$$$$$%88888888888$$$$$$$88888+O+8+;@888++$$$$$%8888@;;;7O+o,o;;;@888",
"8+q;88q7O888888888888$$$$$$$$$$$$$$$$$6+8888888$$$$$$$e888%$$$$$$e88O;7.2:888888:88888%$$$$$$$$$$$$$$$$$$$$%88888888888$$$$$$$88888888+o;@88888$$$$$%888O#q8@,+8Oq#77;O888",
"88o#o@#q8888888888888$$$$$$$$$$$$$$$$6+88888888$$$$$$$e888>$$$$$$>888@ $$$$$$$$$$+8888%$$$$$$$$$$$$$$$$$$$$%88888888888$$$$$$$888888+oq#q+888886$$$$>888O,@8o#O88++O@7+888",
"+8Oq,#;@8888888888888$$$$$$$$$$$$$$$6:888888888$$$$$$$.++8e$$$$$$688888:$$$$$$$3888888%$$$$$$$$$$$$$$$$$$$$%88888888888$$$$$$$888888@7#o8888888%$$$$2888O;7o7;@888+@;o8888",
"888O@@+88888888888O@O$$$$$$$6$$$$$$$$=O8888888+$$$$$$$0;7@:$$$$$$$.88888$$$$$$$>8888@8%$$$$$$6.e<<%3$$$$$$$%8888888888@$$$$$$$888888+O+88888888%$$6*0O88+o,,;##@8+@7qO8888",
"88888888888888888+q,0$$$$$$6q%$$$$$$$$>8888888e$$$$$$$-@;7+6$$$$$$6+8888%$$$$$$e88o;;O%$$$$$$388@;;79$$$$$$%88888888Oq<$$$$$$$8888888888888888888@7#;+888O#78O;@8O;q+88888",
"8888888888888888+7;@e$$$$$$9o+6$$$$$$$$6:e+888e$$$$$$$28o7+.$$$$$$$6:888%$$$$$$e@;;q@+6$$$$$$3888oq73$$$$$$%8888888Oq70$$$$$$$88888888888888+888+=0@;@888O,@8@;@8++8888888",
"8888888888888888@7@8e$$$$$$$q8+6$$$$$$$$$$e888e$$$$$$$2O7#++$$$$$$$$$$3%6$$$$$$e@;@O++$$$$$$$38888+8%$$$$$$38888888+o@e$$$$$$$888888888888+Oq7@>$$$$wq888O;7q7q+8888888888",
"8888888888+8888+o78o>$$$$$$$o88+$$$$$$$$6:8888.$$$$$$$X#,;q8.$$$$$$$$$$$$$$$$$$e@7@q;7$$$$$$$3888888%$$$$$$$888888888oX$$$$$$$:88888888888O;;79$$$$$$wO88+@;;#O88888888888",
"888888888Oo7qO88##;;9$$$$$$$5888e$$$$$$%+8888o4$$$$$$$*;@o,O83$$$$$$$$$$$$$$$$$eO7,#q;$$$$$$$$888888$$$$$$$$88888888+qw$$$$$$$e8888888888+q7+ $$$$$$$$.O+88OO88888888888++",
"88888888+q,q;o++q,q@$$$$$$$$28888%$$$$%88888+#w$$$$$$$*O8@#O8+6$$$$$$$$$$$$$$$$e8oq@+=$$$$$$$$888888$$$$$$$$:88888888+%$$$$$$$e8888888888+;q8%$$$$$$$$&7o+8888888888888+o#",
"88888888@#@+o#88q;2%$$$$$$$$$%:88+6$$e8888888@3$$$$$$$9O@#7+88+3$$$$$$$$$$$$$$$e8OO881$$$$$$$$+8888+$$$$$$$$e888888888$$$$$$$$>888888888887;7<$$$$$$$$-7@88888888888888o,q",
"88888888o;++77O8@;=%1X.eeeeeee:8883$%8888888:%$$$$$$$$$X#;@88888>$$$$$$$$$$$$$>88888$$$$$$$$$$$6886$$$$$$$$$$3888888:.$$$$$$$$3e+888888888O,,;6$$$$$$9@+88888888888888+#o8",
"88888888@;7q;;o88;7q#q+88888888888$%88888888e%%%%9=e>%3$w@8888888+.3$$$$$$$6>+8888@70<2:888eeeee8oX0e2=:+10ee.8888886$$6%%%3$$$$e888888888+77+:w$$$$%+8888888888888o#O@qO8",
"88888888+o,,o77O8@q77O888888888888:8888888888888+#o88888888888888888@< eee8888888+q77O888888888o,;;q+++8O;@888888888:+88888888888888888888O;q8@;5ee88888888888888o;,;O8O+8",
"888888888@;@8@#O88+O8888888888888888888888888888O#o88888888888888888@;O88888888888+O8888888888;;o@778888o#oq88888888888888888888888888888887#777O88888888888888O;;qo;@888+",
"888888888o;+Oq#O888888888888888888888888888888888q@88888888888888888o#+88888888888888888888+88o#+87o+88+qo@,O888888888888888888O@O888888888Oq##O888888888888888@#oO@#+888@"
};

/* XPM data for 'WRONG!' */
static gchar *wrong_xpm[] = {
/* width height ncolors chars_per_pixel */
"170 30 32 1",
/* colors */
"  c #B57E7D",
". s mask c none",
"X c #1F1D1B",
"o c #4E3232",
"O c #984849",
"+ c #9B1917",
"@ c #FE0305",
"# c #717171",
"$ c #DC3432",
"% c #CA1C1D",
"& c #BD6A6C",
"* c #B31E1D",
"= c #714B4C",
"- c #B80E0E",
"; c #873233",
": c #C55B5C",
"> c #111111",
", c #D00A0A",
"< c #AC3031",
"1 c #585858",
"2 c #484848",
"3 c #9A5B5A",
"4 c #987172",
"5 c #980C0C",
"6 c #303030",
"7 c #741413",
"8 c #D54644",
"9 c #5B0E0E",
"0 c #E92123",
"q c #868687",
"w c #AF4646",
"e c #792728",
/* pixels */
"........................q#>>X#...............................X>1#62.q662q.............#q.61.q61.................................q>q....qq.........2>X6#......61...........",
".......................q1X>>#................................66..61..qq.............#X>1.#q..61.................................16q..............16qqX1.....q61...........",
"............q..........q61.>1..8&..........::&&.. &&&&&:&&&..X1#16#............&8$%-5e>#....#6......$&..........................#Xq.... &$$$$8& .q<&qX1:&&..47O&:$8.......",
"...........#22. 0@...... ..12q.0@:.........8@@@@@@@@@@@@@@@@@<XXX2q......... $@@@@@@@@@<....6221....@@&.........@@0$$$:&:$$@....#1...8@@@@@@@@@@@$@w6X>%0@@@@@@@@@&.......",
"..........#XXw@@@ .....$@...q...0@0 .........$@@@@@@@@@@@@@@@@@=#..........8@@@@@@@@@@@@@:.#612>....@@@&........ %@@@@@@@@8 .......:@@@@@@@@@@@@@@@.21663@@@@@@@@&........",
".........qe%@@@@0......@@:......@@@@0 .......8@@@@@@@@@@@@@@@@@0..........@@@@@@@@@@@@@@@@$2>662q...0@@@$......#66@@@@@@@@........$@@@@@@@@@@@@@@@0...#>4@@@@@@@@ ........",
".....q.80@@@@@@@@.....:@@@..... @@@@@@@:.....&@@@@@@@@@@@@@@@@@@@........@@@@@@@@@@@@@@@@@@-X61.....$@@@@0...q2661@@@@@@@@.......$@@@@@@@@@@@@@@@@$..q66q@@@@@@@@.........",
"...q16#::@@@@@@@@&....@@@@:....8@@@@@@@@@8...&@@@@@@@@@@@@@@@@@@@&......0@@@@@@@@@@@@@@@@@@@<q......$@@@@@@&.q21qq-@@@@@@$......&@@@@@@@@@@@@@@@@@8q#261.@@@@@@@@.........",
"...2>2...*@@@@@@@$...&@@@@@....@@@@@@@@0.....O@@@@@@@@@@@@@@@@@@@$.....:@@@@@@@@@@@@@@@@@@@@@.......$@@@@@@@8.qq6>-@@@@@@$......@@@@@@@@0$&e95%@@@&X6X1..$@@@@@@$q#1q.....",
"..#6#....o@@@@@@@@...0@@@@@8.. @@@@@@@@ .....O@@@@@@@&..:@@@@@@@@$...##0@@@@@@@@@@@@@@@@@@@@@8...q..$@@@@@@@@04>6#$@@@@@@$.....:@@@@@@@$...16q.#9@&q#q...:@@@@@@$6>X6#....",
"..62.q...#7@@@@@@@:.:@@@@@@@&.0@@@@@@@8......&@@@@@@@....$@@@@@@@$..q6e@@@@@@@@@8 . 8@@@@@@@@@..q2X1$@@@@@@@@@@wq.$@@@@@@$.....@@@@@@@-21..2Xq#62@ ......&@@@@@@;6##62....",
"..62262..#X@@@@@@@0.@@@@@@@@@:@@@@@@@@........@@@@@@@....@@@@@@@@:.q#X-@@@@@@@@.......@@@@@@@@ q6X26%@@@@@@@@@@@0.&@@@@@@$.....@@@@@@@o6>q.16X66#:........@@@@@@72..66#...",
".qXX62X1.q17@@@@@@@0@@@@@@@@@@@@@@@@@8........@@@@@@@...&@@@@@@@@ .#6>,@@@@@@@........:@@@@@@@&1X..6%@@@@@@@@@@@@@0@@@@@@:....&@@@@@@%q#61..#26#..........@@@@@@o1.qX>q...",
"..X>#q26...#@@@@@@@@@@@@@@@@@@@@@@@@@.........@@@@@@@..:@@@@@@@@$..q2X@@@@@@@$.........@@@@@@@$66q#6-@@@@@@@@@@@@@@@@@@@@&....&@@@@@@&.qX#...............#,@@@@@1>1XXX#...",
".q2X..22q...:@@@@@@@@@@@@@@@@@@@@@@@8.........@@@@@@@@@@@@@@@@@@....qX@@@@@@@&.........$@@@@@@$#>X2>-@@@@@@@@@@@@@@@@@@@@&....$@@@@@@&q1X#.&& ...... ...q6%@@@@@q2662X#...",
"..161#62.....@@@@@@@@@@@@@@@@@@@@@@@ .........@@@@@@@@@@@@@@@@@......#@@@@@@@:.........0@@@@@@$.6>X1-@@@@@@@@@@@@@@@@@@@@:....8@@@@@@*262. @@@@@@@@@@...q><@@@@@..qq#2q...",
"..q2>6X#..... @@@@@@@@@@@@@@@@@@@@@0.q........@@@@@@@@@@@@@@@@........@@@@@@@$.........@@@@@@@$.1X..%@@@@@@@@@@@@@@@@@@@@%q...&@@@@@@@61... @@@@@@@0#...qXe@@@@$...#X1....",
"...q##q.......0@@@@@@@@@@@@@@@@@@@@+61q.......@@@@@@@0@@@@@@@@&.......@@@@@@@@ .....#.:@@@@@@@&.66q#,@@@@@@0@@@@@@@@@@@@@-#... @@@@@@@:.....@@@@@@@52q...19@@@-w..#22q....",
".............. @@@@@@@@@38@@@@@@@@@X61....... @@@@@@06-@@@@@@@@8......$@@@@@@@@...1>X3@@@@@@@@ .#XX6@@@@@@@.&@@@@@@@@@@@@$qq...@@@@@@@@.....$@@@@@@eX....q62.qX#.qX2q.....",
"...............@@@@@@@@$..0@@@@@@@+2q........&@@@@@@%X#0@@@@@@@@@ &...:@@@@@@@@@%59*0@@@@@@@@@...126@@@@@@@...$@@@@@@@@@@%>#...:@@@@@@@@ ...$@@@@@@OX#...q>#&O9#..q.......",
"...............:@@@@@@@... @@@@@@@3..........&@@@@@@@X#4@@@@@@@@@@@&...@@@@@@@@@@@@@@@@@@@@@@w......@@@@@@@....&@@@@@@@@@-1q....@@@@@@@@@@0$@@@@@@@322...q7@@@@0..........",
"................@@@@@@:....+@@@@@@...........:@@@@@@@>X6>@@@@@@@@@ ....8@@@@@@@@@@@@@@@@@@@@-#......@@@@@@@......0@@@@@@@0......:@@@@@@@@@@@@@@@@@@&22q..&@@@@@@@.........",
".........q162q..+@@@@,....q2@@@@@:...........%@@@@@@@=>X#;@@@@@@$.......@@@@@@@@@@@@@@@@@@@06q.....&@@@@@@@&......:@@@@@@@.......0@@@@@@@@@@@@@@@@@&1Xqqq0@@@@@@@$........",
".........2>2X1..o@@@@oq....23@@@@............,@@@@@@@O>#.#-@@@@$.....#...@@@@@@@@@@@@@@@@@@4#......&@@@@@@@&........0@@@@@........@@@@@@@@@@@@@@@@@&q>66=@@@@@@@@0.....q16",
"........#6#.16..2,@@%6q......@@@0.......... $@@@@@@@@@-3#6o0@@&......2#...@@@@@@@@@@@@@@@$.........8@@@@@@@0.........:@@@@ ........0@@@@@@@@@@@@@@@:2XX6#@@@@@@@@$.....1>2",
"........1>q.66q.#7@@=>q......:@@&..........:$::&&e;&&:996X#0@$......q6#....8@@@@@@@@@@@0:........$@@$%-$$%@@@8.........@@@&.........8@@@@@@@@@@@@@8266#..8@@@@@@@.....q61.",
"........#X62XX1..X@*62........@@................qX1...#12#.@$........6#......8@@@@@@@,w .........1>2.##..22. ...........@@:...........:0@@@@@@@@* .##.....:@@@@0...16##2q.",
"........q1>>166#.#;o6q......... ................q61........ ........#6q.........&&;eoq.........1>X>2....qX#..............&............... :7e&#>1...........&&...1>>>q.q..",
".........#X#.#6q...q............................q61.................#Xq...........qq..........XX1#26....1612...............................66662q..............qXX21X#....",
".........1>qq26q.................................2#.................16........................16q.61....21#>q..................q#q.........q266#...............#61q#6....#"
};
